/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm;

import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.spi.project.SubprojectProvider;
import org.openide.filesystems.*;
import org.openide.util.Mutex;

/**
 * SubprojectProvider for Farm projects Loops through subfolders of a farm
 * project and attempts to create projects for them.
 *
 *
 * @author Timon Veenstra
 */
public class FarmSubProjectProvider implements SubprojectProvider {

    private final FarmProject project;
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.farm");
    private final Map<ChangeListener, ProjectDirListener> listeners = new HashMap<>();

    FarmSubProjectProvider(FarmProject project) {
        this.project = project;
    }

    @Override
    public Set<? extends Project> getSubprojects() {
        Set<Project> projects = loadProjects(project.getProjectDirectory());
        return projects;
    }

    private Set<Project> loadProjects(final FileObject dir) {
        return (Set) ProjectManager.mutex().readAccess(new Mutex.Action() {
            @Override
            public Object run() {
                Set newProjects = new HashSet();
                try {
                    FileObject[] subFolders = dir.getChildren(); //subfolders of root folder
                    for (FileObject subFolder : subFolders) {
                        LOG.log(Level.FINEST, "found child of root: {0}", subFolder.getName());
                        if (subFolder.isFolder()) {
                            Project subp = ProjectManager.getDefault().findProject(subFolder);
                            if (subp != null) {
                                newProjects.add(subp);
                            }
                        }
                    }
                } catch (IOException e) {
                    LOG.info(e.getMessage());
                }
                return Collections.unmodifiableSet(newProjects);
            }
        });
    }

    @Override
    public final void addChangeListener(final ChangeListener l) {
        ProjectDirListener local = new ProjectDirListener(l);
        listeners.put(l, local);
        project.getProjectDirectory().addFileChangeListener(local);
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
        ProjectDirListener local = listeners.get(l);
        if (local != null) {
            listeners.remove(l);
            project.getProjectDirectory().removeFileChangeListener(local);
        }
    }

    /**
     * Wrapper for a generic ChangeListener to be able to listen to FileChanges
     *
     */
    private class ProjectDirListener implements FileChangeListener {

        private final ChangeListener changeListener;

        public ProjectDirListener(ChangeListener changeListener) {
            this.changeListener = changeListener;
        }

        @Override
        public void fileFolderCreated(FileEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }

        @Override
        public void fileDataCreated(FileEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }

        @Override
        public void fileChanged(FileEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }

        @Override
        public void fileDeleted(FileEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }

        @Override
        public void fileRenamed(FileRenameEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }

        @Override
        public void fileAttributeChanged(FileAttributeEvent fe) {
            changeListener.stateChanged(new ChangeEvent(FarmSubProjectProvider.this));
        }
    }
}
